/**
 * @file     args.js
 * @description
 *  Windowsのコマンドライン引数を処理する.
 *  HTA用.
 *
 * @version  0.2
 * @date     2010-02-27
 * @auther   Masashi N.
 * @licence  NYSL Version 0.9982
 *            http://www.kmonos.net/nysl/
 *            http://ja.wikipedia.org/wiki/NYSL
 * @history
 * 0.2 2010-02-27
 *   Argument.interpret() の仕様を変更.
 * 0.1 2008-08-27
 *   初版.
 */

;(function(window){
	var _Argument = window.Argument,
		Argument = window.Argument = {};

	Argument.noConflict = function()
	{
		window.Argument = _Argument;
		return Argument;
	};

	//@param argv  an array
	Argument.parseCommandLine = function(cmdlinestr)
	{
		cmdlinestr = cmdlinestr.replace(/^\s+|\s+$/g, '');
		var inquote = false;
		var argc = 0;
		var argv= new Array();
		argv[argc] = '';
		for (var i = 0, c; c = cmdlinestr.charAt(i); i++) {
			if (inquote) {
				argv[argc] += c;
				if (c == '"') {
					inquote = false;
				} else if (c == '\\') {
					c = cmdlinestr.charAt(++i);
					if (c == '') break;
					argv[argc] += c;
				}
			} else {
				if (c == ' ' || c == '\t') {
					argv[++argc] = '';
					c = cmdlinestr.charAt(i + 1);
					while (c == ' ' || c == '\t') {
						i++;
						c = cmdlinestr.charAt(i + 1);
					}
				} else {
					argv[argc] += c;
					if (c == '"') {
						inquote = true;
					}
				}
			}
		}
		return argv;
	};

	//@param argv      argument values, the first one of which is a path of the executable.
	//@param switches  an array representing a mapping from switch string regexp ('p' property) to callback functions ('c' property)
	//@param noswitch  default callback when no switch matches
	Argument.interpret = function(argv, switches)
	{
		if (typeof (argv) == 'string') {
			argv = this.parseCommandLine(argv);
		}
		var slen = switches.length || 0;
		for (var i = 1, len = argv.length; i < len; i++) {
			var argi = argv[i];
			for (var j = 0; j < slen; j++) {
				var match = switches[j].p.exec(argi);
				if (match) {
					if (!switches[j].c.call(this, match, i)) {
						break;
					}
				}
			}
		}
	};

	//@param doc   a document object creating HTML elements
	//@param argv  an array of argument values
	Argument.toHTML = function(doc, argv)
	{
		var ol = doc.createElement('ol');
		ol.className = 'arguments';
		for (var i = 0; i < argv.length; i++) {
			var li = doc.createElement('li');
			li.appendChild(doc.createTextNode(argv[i]));
			ol.appendChild(li);
		}
		return ol;
	};
})(window);
