/**
 * @file     csview.js
 * @description
 *           csview のメインスクリプト.
 * @author   Masashi Nakanishi
 * @license  MIT License  http://www.opensource.org/licenses/mit-license.php
 */
;(function(window, $, App){
	window.document.title = App.applicationName + " - v" + App.version;

	var messages = {
		'button.openfile.hint': "ファイルを開く",
		'button.sql.hint': "SQL クエリ",
		'button.selectcolumns.hint': $.jgrid.col.caption,
		'button.savecsv.hint': "保存",
		'dialog.openfile.caption': "ファイルを開く",
		'dialog.openfile.filter': "CSV ファイル (*.csv)|*.csv|すべてのファイル (*.*)|*.*|",
		'dialog.savecsv.caption': "ファイルを保存する",
		'dialog.savecsv.guidetext': "保存先のファイル名 (*.csv) を入力してください",
		'dialog.savecsv.error.fileexist': "ファイル {0} はすでに存在します",
		'dialog.sql.caption': "SQL クエリ",
		'dialog.sql.guidetext': "列名に特殊な文字が含まれる場合は [ ] で囲んでください.",
		'dialog.sql.label.execute': "更新",
		'dialog.sql.label.help': "？",
		'dialog.sqlhelp.caption': "ヘルプ",
		'dialog.sqlhelp.text1': "テーブル名, カラム名などに以下の文字を使用した場合, 置き換えられます",
		'error.table.addview.already': 'already set a view',
		'error.table.open.already': 'already opened any data source',
		'error.view.load.colname': '"{0}" cannot be used as column name',
		'error.illegalarg': 'illegal argument',
		'error.notsupport': 'not supported'
	},

	document = window.document,
	manager = window.manager = new function(){
		var p = {
			tables: []
		};

		this.close = function(o) {
			if (typeof(o) == 'undefined') {
				$.each(p.tables, function(){
					this.close();
				});
				p.tables = [];
			} else if (o instanceof this.Table) {
				o.close();
				p.tables = $.grep(p.tables, function(e, i){ return e !== o; });
			} else if (o instanceof this.View) {
				var table = o.getTable();
				if (table) {
					table.close(o);
					if (table.getViews().length == 0) {
						this.close(table);
					}
				}
			} else if (typeof(o) == 'number') {
				var table = p.tables[o];
				if (table) {
					this.close(table);
				}
			} else {
				throw new Error(messages['error.illegalarg']);
			}
		};
		this.open = function(path, id) {
			var table = new this.Table();
			p.tables.push(table);
			table.open(path);
			var view = new this.View(table, id);
			view.load();
			return view;
		};
		this.activeTable = function() {
			return p.tables[0] || null;
		};
		this.activeView = function() {
			var table = this.activeTable();
			if (table) {
				var views = table.getViews();
				return views[0] || null;
			}
			return null;
		};

		return this;
	};

	$(window).bind('beforeunload', function(){ manager.close(); });

	manager.Table = function(){
		var p = {
			dataSource: null,
			filepath: null,
			directory: null,
			filename: null,
			extension: null,
			views: [],
			headerDefault: [
				{ name: 'A', align:'center' },
				{ name: 'B', align:'center' },
				{ name: 'C', align:'center' },
				{ name: 'D', align:'center' },
				{ name: 'E', align:'center' },
				{ name: 'F', align:'center' }
			],
			dataDefault: '{' +
				'"total":1,' +
				'"page":1,' +
				'"records":6,' +
				'"rows":['+
					'{"id":"1","cell":["c","","","","",""]},' +
					'{"id":"2","cell":["","s","","","",""]},' +
					'{"id":"3","cell":["","","v","","",""]},' +
					'{"id":"4","cell":["","","","i","",""]},' +
					'{"id":"5","cell":["","","","","e",""]},' +
					'{"id":"6","cell":["","","","","","w"]}]' +
				'}'
		};

		this.getViews = function() {
			return p.views.slice(0);
		};
		this.addView = function(view) {
			if (p.views.length) {
				throw new Error(messages['error.table.addview.already']);
			}
			p.views.push(view);
		};
		this.getDirectory = function() {
			return p.directory;
		};
		this.getSourceName = function() {
			if (p.dataSource && p.dataSource.supports('getSourceName')) {
				return p.dataSource.getSourceName();
			}
			return p.filename;
		};
		this.open = function(path) {
			if (path.length >= 2 && path.charAt(0) == '"' && path.charAt(path.length - 1) == '"') {
				path = path.substring(1, path.length - 1);
			}
			p.filepath = path;
			var sep = path.lastIndexOf('\\');
			p.directory = (sep >= 0) ? path.substring(0, sep) : '.';
			var fname = p.filename = (sep >= 0) ? path.substring(sep + 1) : path;
			var dot = fname.lastIndexOf('.');
			var ext = p.extension = (dot >= 0) ? fname.substring(dot + 1).toLowerCase() : '';
			if (ext == 'csv') {
				if (p.dataSource) {
					throw new Error(messages['error.table.open.already']);
				}
				p.dataSource = new $.DataSource.CSVDB(path);
			}
		};
		this.close = function(v) {
			if (typeof(v) == 'undefined') {
				$.each(p.views, function(){
					this.close();
				});
				p.views = [];
				if (p.dataSource) {
					p.dataSource.close();
					p.dataSource = null;
				}
			} else if (v instanceof manager.View) {
				v.close();
				p.views = $.grep(p.views, function(e, i){ return e !== v; });
			} else {
				throw new Error(messages['error.illegalarg']);
			}
		};
		this.setRecordsParams = function(pagesize, currentpage, sortindex, sortorder) {
			var ds = p.dataSource;
			if (ds) {
				ds.setPageSize(pagesize);
				ds.setCurrentPage(currentpage);
				ds.setSort([{ column: sortindex, order: sortorder }]);
			}
		};
		this.setQuery = function(sql) {
			if (p.dataSource && p.dataSource.supports('setQuery')) {
				p.dataSource.setQuery(sql);
				$.each(p.views, function(){
					this.load(); // to reset colModel
				});
			} else {
				throw new Error(messages['error.notsupport']);
			}
		};
		this.header = function() {
			return p.dataSource ? p.dataSource.header() : p.headerDefault;
		};
		this.json = function() {
			if (p.dataSource) {
				return p.dataSource.json();
			}
			return p.dataDefault; // '{}';
		};
		this.getQuery = function() {
			if (p.dataSource && p.dataSource.supports('getQuery'))
				return p.dataSource.getQuery();
			throw new Error(messages['error.notsupport']);
		};
		this.saveCsv = function(path) {
			if (p.dataSource && p.dataSource.supports('saveCsv')) {
				p.dataSource.saveCsv(path, window);
			} else {
				throw new Error(messages['error.notsupport']);
			}
		};
		this.supports = function(method){
			return p.dataSource ? p.dataSource.supports(method) : false;
		};

		return this;
	};

	manager.View = function(table, id){
		var thisView = this;
		var p = {
			autoAdjust: true,
			gridParam: {
				url: '.', // expected to return the empty string
				datatype: 'json',
				//colNames: header.names,
				//colModel: View._toColModel(header),
				rowNum: 20,
				rowList: [ 10, 20, 30, 50, 100, 200, 300, 500, 1000 ],
				pager: "#pager",
				//sortname: header[0],
				//sortorder: "asc",
				gridview: true,
				viewrecords: true,
				autowidth: true,
				rownumbers: true,
				loadError: function(xhr, status, error) {
					thisView.adjustSize();
				},
				loadComplete: function(xhr) {
					thisView.adjustSize();
				},
				serializeGridData: function(postData) { // データソースにリクエストパラメタをセットするために serializeGridData を利用
					var prmNames = $(this).jqGrid('getGridParam', 'prmNames');
					table.setRecordsParams( postData[prmNames.rows], postData[prmNames.page],
							postData[prmNames.sort], postData[prmNames.order] );
					return postData;
				},
				ajaxGridOptions: {
					cache: false,
					dataFilter: function(data, type) { // データソースにアクセスして json データを引き出す. ajax のレスポンスを強制上書き.
						return table.json();
					}
				}
			},
			toColModel: function(header) {
				var ret = [];
				for (var i = 0, n = header.length; i < n; i++) {
					var name = header[i].name;
					if (name == "subgrid" || name == "cb" || name == "rn") {
						throw new Error(messages['error.view.load.colname'].replace("{0}", name));
					}
					var align = header[i].align ? header[i].align : (header[i].type == 'integer' || header[i].type == 'decimal' ? 'right' : 'left');
					ret.push({ name: name, index: name, align: align });
				}
				return ret;
			}
		};

		table.addView(this);

		this.getTable = function() {
			return table;
		};
		this.load = function() {
			var header = table.header();
			this.close();
			this.startAdjust();
			var $grid = $(id).jqGrid($.extend(p.gridParam, { colModel: p.toColModel(header) /*, sortname: header[0].name */ }))
				.jqGrid("navGrid", "#pager", { view:true, search:false, add:false, edit:false, del:false, viewicon:'ui-icon-note', refreshstate:'current' },
					{}, {}, {}, { multipleSearch:true, closeOnEscape:true }, { closeOnEscape:true })
				.jqGrid("navButtonAdd", "#pager", { caption:'', title:messages['button.selectcolumns.hint'], buttonicon:"ui-icon-circlesmall-minus",
					onClickButton: function(){
						//$(id).jqGrid('setColumns', {
						//	width: 300,
						//	colnameview: false
						//});
						$(id).jqGrid('columnChooser');
					}
				})
				.jqGrid("navButtonAdd", "#pager", { caption:'', title:messages['button.openfile.hint'], buttonicon:"ui-icon-folder-open", position:"first",
					onClickButton: function(){
						var path = HtmlDlgHelper.openfiledlg("", "", messages['dialog.openfile.filter'], messages['dialog.openfile.caption']);
						var p = path.indexOf('\0');
						if (p >= 0) {
							path = path.substring(0, p);
						}
						if (path == "") {
							return;
						}
						manager.close();
						manager.open(path, '#grid1');
					}
				});
			if (table.supports('setQuery') && table.supports('getQuery')) {
				if (table.supports('saveCsv')) {
					$grid.jqGrid("navButtonAdd", "#pager", { caption:'', title:messages['button.savecsv.hint'], buttonicon:"ui-icon-disk", position:"first",
						onClickButton: function(){
							var filename = window.prompt(messages['dialog.savecsv.guidetext'], "");
							if (filename == null || filename == "")
								return;
							if (!/\.csv$/i.test(filename)) {
								filename += ".csv";
							}
							if (new ActiveXObject("Scripting.FileSystemObject").FileExists(table.getDirectory() + "\\" + filename)) {
								window.alert(messages['dialog.savecsv.error.fileexist'].replace("{0}", filename));
								return;
							}
							table.saveCsv(filename);
						}
					});
				}
				$grid.jqGrid("navButtonAdd", "#pager", { caption:'', title:messages['button.sql.hint'], buttonicon:"ui-icon-wrench", position:"first",
					onClickButton: function(){
						$("textarea", "#querydialog").val(table.getQuery());
						$("#querydialog").dialog("option", "width", Math.min(Math.max($(document).width() / 2, 400), $(document).width()))
							.dialog("open");
					}
				});
			}
			var sname = table.getSourceName();
			if (sname != null && sname != "") {
				document.title = sname + " - " + App.applicationName;
			}
		};
		this.reload = function() {
			$(id).trigger('reloadGrid');
		};
		this.adjustSize = function() {
			if (!p.autoAdjust) return;
			var h = $("html").height() - $(".ui-jqgrid-hdiv").height() - $("#pager").height() - 4; // 4px for 4 horizontal borders
			var w = $("html").width() - 2; // 2px for 2 vertical borders
			$(id).jqGrid("setGridHeight", h > 0 ? h : 0).jqGrid("setGridWidth", w > 64 ? w : 64); // error occurs when less than 64. why?
		};
		this.stopAdjust = function() {
			p.autoAdjust = false;
		};
		this.startAdjust = function() {
			p.autoAdjust = true;
		};
		this.close = function() {
			this.stopAdjust();
			$(id).jqGrid("GridUnload");
		};

		return this;
	};

	$(function(){
		$("body")
			.append('<table id="grid1"></table>')
			.append('<div id="pager"></div>')
			.append('<div class="dialog" id="querydialog"><p>' + messages['dialog.sql.guidetext'] + '</p><textarea rows="5" cols="80"></textarea></div>')
			.append('<div class="dialog" id="help_specialchars"><p>' + messages['dialog.sqlhelp.text1'] + '</p><table><tr><td>.</td><td>→</td><td>#</td></tr><tr><td>\\</td><td>→</td><td>_</td></tr></table></div>')
			.append('<object id="HtmlDlgHelper" classid="CLSID:3050f4e1-98b5-11cf-bb82-00aa00bdce0b"></object>');

		var buttons = {};
		buttons[messages['dialog.sql.label.help']] = function(){
			$("#help_specialchars").dialog({
				title: messages['dialog.sqlhelp.caption']
			});
		};
		buttons[messages['dialog.sql.label.execute']] = function(){
			var active = manager.activeTable();
			if (active) {
				active.setQuery($("textarea", this).val());
			}
			$(this).dialog('close');
		};
		var resizeTextarea = function(event, ui) {
			var t = $('textarea', this);
			var p = $(this).dialog("widget");
			var h = p.height() - 150;
			var w = p.width() - 40;
			t.width(w).height(h);
		};
		$("#querydialog").dialog({
			autoOpen: false,
			modal: true,
			title: messages['dialog.sql.caption'],
			minHeight: 230,
			minWidth: 400,
			buttons: buttons,
			open: resizeTextarea,
			resizeStart: function(event, ui) { $('textarea', this).hide(); },
			resize: resizeTextarea,
			resizeStop: function(event, ui) { $('textarea', this).show(); },
			close: function(event, ui) { $("#help_specialchars").dialog('close'); }
		})

		var loaded = false;
		Argument.interpret(App.commandLine, [
		    // switches
			//{ p:/^-hr=(\d+)$/i,                                    c:function(match) { manager.header.rows = Number(match[1]); } },
			//{ p:/^-hc=(\d+)$/i,                                    c:function(match) { manager.header.cols = Number(match[1]); } },
			//{ p:/^-ir=(1|a|A|i|I|α|Α|あ|ア|い|一|壹)$/i,         c:function(match) { manager.index.rows = match[1]; } },
			//{ p:/^-ic=(1|a|A|i|I|α|Α|あ|ア|い|一|壹)$/i,         c:function(match) { manager.index.cols = match[1]; } },
			//{ p:/^(?:-e|--editable)=(yes|no|on|off|true|false)$/i, c:function(match) { manager.editable = /yes|on|true/i.test(res[1]); } },
			{
				p: /.+/,  // non-switches
				c: function(match) {
					if (!loaded) {
						manager.open(match[0], '#grid1');
						loaded = true;
					}
				}
			}
		]);
		if (!loaded) {
			manager.open('', '#grid1');
		}

		$(window).resize(function(){
			var view = manager.activeView();
			if (view) {
				view.adjustSize();
			}
		}).resize();
	});
})(window, jQuery, App);
